/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Location.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomLocation.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomLocation.h"
#include "gdom/PrivateIntGdomLocation.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::Location* obj)
{
    return WebCore::cacheDOMObject<WebCore::Location, GDOMLocation>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMLocation::GDOMLocation(PassRefPtr<WebCore::Location> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMLocation::~GDOMLocation()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}


/* XXX TODO: custom function gdom_location_assign */


/* XXX TODO: custom function gdom_location_replace */


/* XXX TODO: custom function gdom_location_reload */

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "Location.h"
#include <gdom/GdomLocation.h>
#include <gdom/PrivateGdomLocation.h>
#include <gdom/PrivateIntGdomLocation.h>
#include <gdom/GdomLocationprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomLocation, gdom_location, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomLocationPrivate {
    class WebCore::Location *item;
};

#define GDOM_LOCATION_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_LOCATION, GdomLocationPrivate))
*/
G_END_DECLS
WebCore::Location *
gdom_location_get_private(GdomLocation *request)
{
    GDOMLocation *res;
    res = static_cast<GDOMLocation*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::Location*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_HREF,
    PROP_PROTOCOL,
    PROP_HOST,
    PROP_HOSTNAME,
    PROP_PORT,
    PROP_PATHNAME,
    PROP_SEARCH,
    PROP_HASH,
};


static void gdom_location_finalize(GObject* object)
{
    /*GdomLocation* request = GDOM_LOCATION(object);*/
    /*GdomLocationPrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_location_parent_class)->finalize(object);
}

static void gdom_location__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomLocation *self = GDOM_LOCATION(object);
    WebCore::Location * gself = (gdom_location_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_location__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomLocation *self = GDOM_LOCATION(object);
    WebCore::Location * gself = (gdom_location_get_private(self));
    switch (prop_id) {
    case PROP_HREF:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->href()));
         break;
    }
    case PROP_PROTOCOL:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->protocol()));
         break;
    }
    case PROP_HOST:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->host()));
         break;
    }
    case PROP_HOSTNAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->hostname()));
         break;
    }
    case PROP_PORT:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->port()));
         break;
    }
    case PROP_PATHNAME:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->pathname()));
         break;
    }
    case PROP_SEARCH:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->search()));
         break;
    }
    case PROP_HASH:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->hash()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_location_class_init(GdomLocationClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_location_finalize;
    g_klass->set_property = gdom_location__set_property_fn;
    g_klass->get_property = gdom_location__get_property_fn;

    g_object_class_install_property (g_klass, PROP_HREF,
            g_param_spec_string ("href", /* name */
                "location_href", /* short description */
                "read-write  gchar * Location.href", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_PROTOCOL,
            g_param_spec_string ("protocol", /* name */
                "location_protocol", /* short description */
                "read-write  gchar * Location.protocol", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_HOST,
            g_param_spec_string ("host", /* name */
                "location_host", /* short description */
                "read-write  gchar * Location.host", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_HOSTNAME,
            g_param_spec_string ("hostname", /* name */
                "location_hostname", /* short description */
                "read-write  gchar * Location.hostname", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_PORT,
            g_param_spec_string ("port", /* name */
                "location_port", /* short description */
                "read-write  gchar * Location.port", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_PATHNAME,
            g_param_spec_string ("pathname", /* name */
                "location_pathname", /* short description */
                "read-write  gchar * Location.pathname", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_SEARCH,
            g_param_spec_string ("search", /* name */
                "location_search", /* short description */
                "read-write  gchar * Location.search", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));

     g_object_class_install_property (g_klass, PROP_HASH,
            g_param_spec_string ("hash", /* name */
                "location_hash", /* short description */
                "read-write  gchar * Location.hash", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READWRITE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomLocationPrivate));*/
}

static void gdom_location_init(GdomLocation* request)
{
    /*GdomLocationPrivate* priv = GDOM_LOCATION_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

